#include <mega8.h>
#include <math.h>
#include <delay.h>

#define H1 PORTD.5
#define H2 PORTB.0
#define PHW 50
#define ocp_led PORTB.4
#define crg_led PORTB.5
//___________________________________________________________________________________
#define A PORTC.2
#define B PORTC.3                 
#define C PORTC.4
#define D PORTC.5
#define E PORTD.0
#define F PORTD.1
#define G PORTD.2
#define DP PORTD.3

#define D1 PORTD.4
#define D2 PORTB.6
#define D3 PORTB.7

#define ZU PORTB.3
//#define ZUon (OCR2=255)
//#define ZUoff (OCR2=15)
#define AC_switch PORTC.1
#define INV310 PORTC.0
#define INV310on (PORTC.0=1)
#define INV310off (PORTC.0=0)
#define off 0
#define on 1

typedef unsigned char byte;

const unsigned char codes[10]={0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09};
volatile unsigned char data[3]={0x00,0x00,0x00};
volatile unsigned char counter=0;
volatile unsigned int voltage; 
volatile bit ac = 0, dc = 0, irn = 0;
//___________________________________________________________________________________________
 void ZU_on ()
 {
    OCR2 = 255;
    TCCR2 |= (1<<COM21);
    crg_led = 1;
 }
 //___________________________________________________________________________________
  void ZU_off ()
 {
    OCR2 = 0;
    TCCR2 &= (~(1<<COM21));
    crg_led = 0;
 }
//___________________________________________________________________________________
void ZU_stb ()
{
    OCR2 = 17;
    TCCR2 |= (1<<COM21);
    crg_led = 0;
}
//___________________________________________________________________________________
byte sin_table [];  
byte i;

void sin_init (void) { 
 float g=(PI/PHW);
 byte i=0; 
 do { 
 sin_table[i]=(byte)((sin((float)i *g)*(float)245)); 
 i++; 
 } while(i!=PHW); 
 }  
//___________________________________________________________________________________
void watchdog_on ()
{
WDTCR |= ((1<<WDE)|(0<<WDP2)|(1<<WDP1)|(1<<WDP0));  //wdt on with 0.13sec freq
}
//___________________________________________________________________________________
void init_pwm (void)
{
TCCR1A |= ((1<<COM1A1)|(1<<WGM10)); //  
TCCR1A &= (~(1<<COM1B1));
TCCR1B |= (1<<CS10); //= /1
OCR1A=0x00;
OCR1B=0x00;
}
//___________________________________________________________________________________
void init_tim0 ()
{
TCCR0=0x03;
TCNT0=0x00;
TIMSK |= (1<<TOIE0);
}
//___________________________________________________________________________________

//___________________________________________________________________________________
void port_init ()
{
DDRB = 0b11111111;
PORTB = 0x00;

DDRD = 0b00111111;
PORTD = 0x00;

DDRC = 0b00111111;
PORTC = 0x00;

H1 = 0;
H2 = 1;

}
//___________________________________________________________________________________
void clear_display ()
{
 A=1; B=1; C=1; D=1; E=1; F=1; G=1; DP=1; D1=0; D2=0; D3=0;
}
//___________________________________________________________________________________________
void write_code ()
{
switch (data[counter])
   {
      case 0:            (A=0),(B=0),(C=0),(D=0),(E=0),(F=0);           break;
      case 1:            (B=0),(C=0);                                                   break;
      case 2:            (A=0),(B=0),(D=0),(E=0),(G=0);                     break;
      case 3:            (A=0),(B=0),(C=0),(D=0),(G=0);                     break;
      case 4:            (B=0),(C=0),(F=0),(G=0);                               break;
      case 5:            (A=0),(C=0),(D=0),(F=0),(G=0);                     break;
      case 6:            (A=0),(C=0),(D=0),(E=0),(F=0),(G=0);           break;
      case 7:            (A=0),(B=0),(C=0);                                         break;
      case 8:            (A=0),(B=0),(C=0),(D=0),(E=0),(F=0),(G=0); break;
      case 9:            (A=0),(B=0),(C=0),(D=0),(F=0),(G=0);           break;
      default:                                                                                  break;
   }
} 
//___________________________________________________________________________________________
void enable_digit  ()
{
 switch (counter)   
    {
     case 0: (D1 = 1); break;
     case 1: (D2 = 1), (DP=0); break;
     case 2: (D3 = 1); break;
     default:               break;
    }

}
//___________________________________________________________________________________________
void convert_data (unsigned int x)
{
 unsigned int temp,res;
 temp=x;
 res=temp/1000;//Calculate 1000-s
 data[2]=codes[res];
 temp=temp-res*1000;
 res=temp/100;//Calculate 100-s  
 data[1]=codes[res];
 temp=temp-res*100;
 res=temp/10;//Calculaate 10-s
 data[0]=codes[res];
}
 //___________________________________________________________________________________________
void ADC_Init()
{
  ADMUX = (1<<REFS1)|(1<<REFS0)|(1<<ADLAR)|(0<<MUX3)|(1<<MUX2)|(1<<MUX1)|(1<<MUX0);// Vref2,5V, let adgst, cnl7
  ADCSR = (1<<ADEN)|(0<<ADSC)|(0<<ADFR)|(0<<ADIE)|(0<<ADPS2)|(1<<ADPS1)|(1<<ADPS0); //adc on, 
}
 //___________________________________________________________________________________________
 unsigned int get_adc (void) //reed adc results
{
  unsigned int adcTemp = 0;
  unsigned int adcValue = 0;
  unsigned char count = 0;
  
for (count=0; count <= 7; count++) 
{
ADCSR |= (1<<ADSC);//adc run
while (ADCSR.6 != 0x00); //wait for conversion complete
   adcTemp = ADCH; //8bit precision
   adcValue = adcValue + adcTemp;
}
    adcValue = (adcValue >> 3); //average conversion result
    return adcValue;
}
//___________________________________________________________________________________________

interrupt [TIM0_OVF] void timer0_ovf_isr(void)
{
bit tmp1 = H1;
bit tmp2 = H2;
TCNT0 = 0xE7;
OCR1A = sin_table[i];
OCR1B = sin_table[i];
i++;   
if (i>PHW) {i=0, H1 = 0, H2 = 0, OCR1A=0x00, OCR1B=0x00, delay_us (5), H1= ~tmp1, H2= ~tmp2, TCCR1A ^= ((1<<COM1A1)|(1<<COM1B1));}

}
//___________________________________________________________________________________
interrupt [TIM2_OVF] void timer2_ovf_isr(void)
{
#asm("wdr")
}
//___________________________________________________________________________________
void init_ancp (void)
{
ACSR &= (~((1<<ACIS1)|(1<<ACIS0))); //int on toggle
ACSR |= ((1<<ACBG)|(1<<ACIE));   //1,22V ion on, interrupt on
}
//___________________________________________________________________________________
void inv_stop  (void)
{
    H1 = 0; //hi part off
    H2 = 0; //hi part off
    TCCR1A &= (~((1<<COM1A1)|(1<<COM1B1))); // disconnect pwm output
    PORTB &= (~((1<<1)|(1<<2))); //lo part off
    TIMSK &= (~(1<<TOIE0)); //timer0 int off
    TCCR1B &= (~(1<<CS10)); // timer1 stop
    //TCCR0 &= (~((1<<CS00)|(1<<CS01)|(1<<CS02))); //timer0 stop 
    //delay_ms (5000);
     irn = 0; i = 0;
 }
//___________________________________________________________________________________ 
void inv_run (void)
{
    //delay_ms (5000);    
    port_init ();
    init_tim0 ();
    init_pwm ();
    irn = 1;
} 
//___________________________________________________________________________________
interrupt [ANA_COMP] void ana_comp_isr(void)
{
if (ACSR.5 == 0) //overcurrent mode 
   {
    inv_stop ();
    ocp_led = 1; // overcurrent indication on 
    }


if (ACSR.5 != 0) //normal mode 
  {
    delay_ms (5000);
    inv_run ();
    ocp_led = 0;  // overcurrent indication off
    }
} 
//___________________________________________________________________________________
char check_AC ()
{
unsigned char ac220 = 0;
ADMUX &= (~(1<<MUX0));// switch to adc6 
ac220 = get_adc();
ADMUX |= (1<<MUX0);// switch back to adc7
if (ac220 > 127) return 1;
else return 0;
}
//___________________________________________________________________________________
char check_DC ()
{
unsigned char dc12 = 0;
dc12 = get_adc ();
if (dc12 > 166) return 1;
else return 0;
}
//___________________________________________________________________________________ 
void  calculate_voltage ()
{
    voltage = get_adc();
    voltage = ((unsigned long)voltage * 1800)/255; //Vbat calculation
    convert_data (voltage);
}
//___________________________________________________________________________________ 
void charger ()
{
    
     byte Vbat = 0; 
     ac = check_AC ();    
     if  (ac == 1) 
    {
     Vbat = get_adc ();
     if (Vbat < 85) {ZU_off ();}
     else if (Vbat > 226) {ZU_stb ();}
     else if ((Vbat >= 85)&(Vbat < 180)) {ZU_on ();}     
    }
    if (ac == 0)  {ZU_off ();} 
}
//___________________________________________________________________________________ 
void DC310 () 
{
    ac = check_AC ();    
    dc = check_DC ();
    
if ((ac == 1)|(ocp_led ==1))
    {
     AC_switch = 0;
     //delay_ms (1000);
     INV310off;
     if (irn == 1) inv_stop ();    
    }
else if ((dc == 1)&(ac == 0)&(ocp_led ==0))
    {
      if (irn == 0) {inv_run ();}
      INV310on;
      //delay_ms (1000);
      AC_switch = 1;      
    }
else if ((dc == 0)&(ac == 0))
    {
     if (irn == 1) {inv_stop ();}
     AC_switch = 0;
     //delay_ms (1000);
     INV310off;
     
     
    }     
    
}
//___________________________________________________________________________________
void init_tim2 () //watchdog control
{
ASSR=0x00;
TCCR2=0b01101111;
TCNT2=0x00;
//ac = check_AC ();
//if (ac == 1) {OCR2=0x0A; DDRB.3 = 1;}
//else {OCR2=0x00; DDRB.3 = 0; ZU = 0;}
TIMSK |=(1<<TOIE2);
OCR2 = 0;
TCCR2 &= (~(1<<COM21));
//TIMSK |=0x40;
}
//___________________________________________________________________________________ 
void display ()
{
 clear_display ();
 enable_digit ();
 write_code ();
 counter=(counter+1)%3;//Increment digit counter 
}
//___________________________________________________________________________________ 
void sleep ()
{
 AC_switch = 0;
 inv_stop ();
 INV310off;
 ZU_off ();
 clear_display ();
 delay_ms (6000);
}


void main(void)
{
 port_init ();
 AC_switch = 0;
 ZU = 0;
 INV310 = 0;
 sin_init ();
 init_tim0 ();
 init_pwm ();
 init_ancp ();
 ADC_Init ();
 init_tim2 ();
  #asm("sei")
  watchdog_on ();
  irn = 1;
 


while (1)
{ 
    
    ac = check_AC ();    
    dc = check_DC ();
    
    if ((dc==0)&(ac == 0) )
    { sleep ();}
    else if ((dc==1)&(ac== 0)&(irn == 0)&(ocp_led==0))
    {inv_run (); ZU_off ();}
    
    calculate_voltage ();
    display ();
    
    if (ac == 1)            {charger ();} 
    
    if (ocp_led == 0)    {DC310 ();}

}
}